import pytest
from unittest.mock import patch, MagicMock
from cmpparis.mssql import Mssql

class TestMssql:

    @pytest.fixture
    def mssql_instance(self):
        return Mssql(
            server="test_server",
            database="test_db",
            username="test_user",
            password="test_pass"
        )

    def test_init(self, mssql_instance):
        assert mssql_instance.server == "test_server"
        assert mssql_instance.database == "test_db"
        assert mssql_instance.username == "test_user"
        assert mssql_instance.password == "test_pass"
        assert not hasattr(mssql_instance, 'port')

    def test_init_with_port(self):
        odbc = Mssql(
            server="test_server",
            database="test_db",
            username="test_user",
            password="test_pass",
            port=1433
        )
        assert odbc.port == 1433

    @patch('pymssql.connect')
    def test_connect(self, mock_connect, mssql_instance):
        mock_connection = MagicMock()
        mock_connect.return_value = mock_connection

        result = mssql_instance.connect()

        mock_connect.assert_called_once_with(server="test_server", database="test_db", user="test_user", password="test_pass")
        assert result == mock_connection

    @patch('cmpparis.mssql.pymssql.connect', side_effect=Exception('Test exception'))
    def test_connect_exception(self, mock_connect, mssql_instance, capsys):
        result = mssql_instance.connect()
        
        assert result is None
        captured = capsys.readouterr()
        assert "Error connecting to MSSQL server: Test exception" in captured.out

    @patch('cmpparis.mssql.pymssql.connect')
    def test_disconnect(self, mock_connect, mssql_instance):
        mock_connection = MagicMock()
        mock_connect.return_value = mock_connection

        connection = mssql_instance.connect()
        mssql_instance.disconnect(connection)

        mock_connection.close.assert_called_once()

    @patch('cmpparis.mssql.pymssql.connect')
    def test_disconnect_exception(self, mock_connect, mssql_instance, capsys):
        mock_connection = MagicMock()
        mock_connection.close.side_effect = Exception('Test exception')
        mock_connect.return_value = mock_connection

        result = mssql_instance.disconnect(mock_connection)

        assert result is None
        captured = capsys.readouterr()
        assert "Error disconnecting from MSSQL server: Test exception" in captured.out


import io
from office365.runtime.auth.client_credential import ClientCredential
from office365.sharepoint.client_context import ClientContext
from office365.sharepoint.files.file import File as SharePointFile

class Sharepoint:
    def __init__(self, site_url, site_path, client_id, client_secret):
        self.site_url = site_url
        self.site_path = site_path
        self.client_id = client_id
        self.client_secret = client_secret
        self.ctx = ClientContext(f"{self.site_url}/{self.site_path}").with_credentials(ClientCredential(self.client_id, self.client_secret))

    def download_file(self, file_location, local_file_path):
        with open(local_file_path, 'wb') as local_file:
            self.ctx.web.get_file_by_server_relative_url(file_location).download(local_file).execute_query()

    def get_files(self, folder_path):
        folder = self.ctx.web.get_folder_by_server_relative_url(folder_path)
        files = folder.files
        self.ctx.load(files)
        self.ctx.execute_query()
        return files
    
    def read_file(self, file_location):
        response = SharePointFile.open_binary(self.ctx, file_location)

        bytes_file_obj = io.BytesIO()
        bytes_file_obj.write(response.content)
        bytes_file_obj.seek(0)

        return bytes_file_obj
    
    def upload_file(self, folder_path, filename):
        target_folder = self.ctx.web.get_folder_by_server_relative_url(folder_path)

        with open(filename, 'rb') as file_content:
            content = file_content.read()
            target_folder.upload_file(filename, content).execute_query()


import unittest
from unittest.mock import Mock, patch
import io
from cmpparis.sharepoint import Sharepoint

class TestSharepoint(unittest.TestCase):

    def setUp(self):
        self.site_url = "https://example.sharepoint.com"
        self.site_path = "sites/testsite"
        self.client_id = "test_client_id"
        self.client_secret = "test_client_secret"
        self.sharepoint = Sharepoint(self.site_url, self.site_path, self.client_id, self.client_secret)

    @patch('cmpparis.sharepoint.ClientContext')
    @patch('cmpparis.sharepoint.ClientCredential')
    def test_init(self, mock_client_credential, mock_client_context):
        mock_client_context.return_value.with_credentials.return_value = "mocked_context"
        
        sharepoint = Sharepoint(self.site_url, self.site_path, self.client_id, self.client_secret)
        
        mock_client_context.assert_called_once_with(f"{self.site_url}/{self.site_path}")
        mock_client_credential.assert_called_once_with(self.client_id, self.client_secret)
        mock_client_context.return_value.with_credentials.assert_called_once_with(mock_client_credential.return_value)
        self.assertEqual(sharepoint.ctx, "mocked_context")

    def test_get_files(self):
        self.sharepoint.ctx = Mock()
        mock_folder = Mock()
        mock_files = Mock()

        self.sharepoint.ctx.web.get_folder_by_server_relative_url.return_value = mock_folder
        mock_folder.files = mock_files

        result = self.sharepoint.get_files("/test/folder")

        self.sharepoint.ctx.web.get_folder_by_server_relative_url.assert_called_once_with("/test/folder")
        self.sharepoint.ctx.load.assert_called_once_with(mock_files)
        self.sharepoint.ctx.execute_query.assert_called_once()
        self.assertEqual(result, mock_files)

    @patch('cmpparis.sharepoint.SharePointFile.open_binary')
    def test_read_file(self, mock_open_binary):
        mock_response = Mock()
        mock_response.content = b"test content"
        mock_open_binary.return_value = mock_response

        result = self.sharepoint.read_file("/test/file.txt")

        mock_open_binary.assert_called_once_with(self.sharepoint.ctx, "/test/file.txt")
        self.assertIsInstance(result, io.BytesIO)
        self.assertEqual(result.getvalue(), b"test content")

    @patch('builtins.open', new_callable=unittest.mock.mock_open, read_data=b"test content")
    def test_upload_file(self, mock_open):
        self.sharepoint.ctx = Mock()
        mock_folder = Mock()
        
        self.sharepoint.ctx.web.get_folder_by_server_relative_url.return_value = mock_folder

        self.sharepoint.upload_file("/test/folder", "test.txt")

        self.sharepoint.ctx.web.get_folder_by_server_relative_url.assert_called_once_with("/test/folder")
        mock_open.assert_called_once_with("test.txt", "rb")
        mock_folder.upload_file.assert_called_once_with("test.txt", b"test content")
        mock_folder.upload_file.return_value.execute_query.assert_called_once()

if __name__ == '__main__':
    unittest.main()


from .sql_interface import SqlInterface

import pymssql

class Mssql(SqlInterface):
    def __init__(self, server, database, username, password, port = None):
        self.server = server
        self.database = database
        self.username = username
        self.password = password
        if port is not None:
            self.port = port

    def connect(self):
        try:
            return pymssql.connect(server=self.server, user=self.username, password=self.password, database=self.database)
        except Exception as e:
            print(f"Error connecting to MSSQL server: {e}")

    def disconnect(self, connection):
        try:
            connection.close()
        except Exception as e:
            print(f"Error disconnecting from MSSQL server: {e}")



import boto3

from botocore.exceptions import ClientError

def get_secret(secret_name, region_name):

    # Create a Secrets Manager client
    session = boto3.session.Session()
    client = session.client(
        service_name='secretsmanager',
        region_name=region_name
    )

    # In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
    # See XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
    # We rethrow the exception by default.

    try:
        get_secret_value_response = client.get_secret_value(
            SecretId=secret_name
        )
    except ClientError as e:
        if e.response['Error']['Code'] == 'DecryptionFailureException':
            # Secrets Manager can't decrypt the protected secret text using the provided KMS key.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InternalServiceErrorException':
            # An error occurred on the server side.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidParameterException':
            # You provided an invalid value for a parameter.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidRequestException':
            # You provided a parameter value that is not valid for the current state of the resource.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'ResourceNotFoundException':
            # We can't find the resource that you asked for.
            raise e
            
    secret = get_secret_value_response['SecretString']

    return secret


import pandas as pd

from cmpparis.ses_utils import *

supported_extensions = ['csv', 'xlsx']

class File:
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return f"{self.name}"

    def __repr__(self):
        return f"File({self.name})"

    def get_extension(self):
        return self.name.split(".")[-1]

    def get_name(self):
        return self.name

    def csv_to_dataframe(self):
        try:
            #read the file and convert it to a DataFrame
            with open(self.name, 'r', errors="ignore") as csv_file:
                csv_reader = pd.read_csv(filepath_or_buffer=csv_file, delimiter=';', dtype=object)

            return csv_reader
        except FileNotFoundError as e:
            raise Exception(f"The CSV file was not found : {e}")
        except Exception as e:
            raise Exception(f"Error while reading the CSV file : {e}")

    def excel_to_dataframe(self, sheet_name=None):
        try:
            if (sheet_name != None):
                excel_reader = pd.read_excel(io=self.name, sheet_name=sheet_name)
            else:
                excel_reader = pd.read_excel(io=self.name)

            return excel_reader
        except FileNotFoundError as e:
            raise Exception(f"The Excel file was not found : {e}")
        except Exception as e:
            raise Exception(f"Error while reading the Excel file : {e}")

    def extract_message_from_code(self, code):
        msg_list = self.excel_to_dataframe('MESSAGES')

        message = msg_list.loc[msg_list['code'] == code]

        return message['message'].values[0]

    #function that reads the file and sets the content inside a DataFrame according to its extension
    #supported extensions are : csv, xlsx
    #handle exceptions
    def read_file_to_dataframe(self, sheet=None):
        if (self.get_extension() not in supported_extensions):
            raise Exception("Unsupported file extension")

        file_extension = self.get_extension()

        match file_extension:
            case "csv":
                df = self.csv_to_dataframe()

                if (df.size == 0):
                    raise Exception("The dataframe is empty")

                return df
            case _:
                df = self.excel_to_dataframe(sheet)

                if (df.size == 0):
                    raise Exception("The dataframe is empty")

                return df


####################################################
# __init__.py for the 'cmpparis' library
# Created by: Sofiane Charrad
####################################################

from .document_db_manager import DocumentDBManager
from .file import File
from .ftp import FTP
from .parameters_utils import get_parameter
from .quable_api import QuableAPI
from .s3 import S3
from .ses_utils import *
from .sm_utils import *
from .utils import *

__all__ = [
    "DocumentDBManager",
    "File",
    "FTP",
    "QuableAPI",
    "S3",
    "get_secret",
    "send_email",
    "send_email_to_support",
    "get_parameter",
    "format_date",
    "get_current_datetime_formatted",
    "lstrip",
    "remove_diacritics",
    "replace",
    "replace_ampersand",
    "replace_comma",
    "replace_endash",
    "replace_emdash",
    "rstrip",
    "toint",
    "upper",
    "check_email",
    "check_empty_value",
    "check_encoding"
]


import paramiko
import sys

from cmpparis.ses_utils import *

class FTP:
    def __init__(self, host, port=22):
        self.host = host
        self.port = port
        self.transport = paramiko.Transport((self.host, self.port))
        self.sftp = None

    def set_working_directory(self, directory):
        try:
            self.sftp.chdir(directory)
        except Exception as e:
            error_message = f"Error while setting working directory on SFTP server: {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    def login(self, username, passwd):
        try:
            self.transport.connect(username=username, password=passwd)
            self.sftp = paramiko.SFTPClient.from_transport(self.transport)
        except Exception as e:
            error_message = f"Error while connecting to SFTP server: {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    def list_files(self):
        try:
            return self.sftp.listdir()
        except Exception as e:
            error_message = f"Error while listing files on SFTP server: {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    def upload_file(self, localfile, remotefile):
        try:
            self.sftp.put(localfile, remotefile)
        except Exception as e:
            error_message = f"Error while uploading file to SFTP server: {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    def download_file(self, remotefile, localfile):
        try:
            self.sftp.get(remotefile, localfile)
        except Exception as e:
            error_message = f"Error while downloading file from SFTP server: {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    def close(self):
        if self.sftp:
            self.sftp.close()
        if self.transport:
            self.transport.close()



####################################################
# document_db_manager.py for the 'cmpparis' library
# Created by: Sofiane Charrad
####################################################

import os
import pymongo
import sys
import urllib

from bson import ObjectId
from datetime import datetime

class DocumentDBManager:
    def __init__(self, db_user="root", db_pwd="", db_host="localhost", database_name="my_database", collection_name="my_collection", pem_file_path=None):
        self.db_user = db_user
        self.db_pwd = db_pwd
        self.db_host = db_host
        self.database_name = database_name
        self.collection_name = collection_name
        self.pem_file_path = pem_file_path or '/opt/python/utils/global-bundle.pem'
        
        if not os.path.exists(self.pem_file_path):
            self.pem_file_path = os.path.join(os.path.dirname(__file__), 'global-bundle.pem')
        
        self.client = self.connect_to_documentdb()
        self.db = self.client[self.database_name]
        self.collection = self.db[self.collection_name]

    def connect_to_documentdb(self):
        try:
            client = pymongo.MongoClient(
                f"mongodb://{self.db_user}:{self.db_pwd}@{self.db_host}:27017/?tls=true&tlsCAFile={self.pem_file_path}&retryWrites=false&directConnection=true")
            print("Client ok :", client)
            return client
        except Exception as e:
            print("Error while connecting to DocumentDB : ", e)
            sys.exit(1)

    def insert_document(self, document):
        try:
            document['createdAt'] = datetime.now()
            document['lastModificationAt'] = datetime.now()

            result = self.collection.insert_one(document)
            return result.inserted_id
        except Exception as e:
            print(f"Error while inserting data into documentDB : {e}")

    def update_document(self, filter_criteria, update_data):
        try:
            result = self.collection.update_one(filter_criteria, {'$set': update_data})
            return result.modified_count
        except Exception as e:
            print(f"Error while updating data in documentDB : {e}")

    def get_document(self, column, value):
        try:
            return self.collection.find_one({column: value})
        except Exception as e:
            print(f"Error while getting data from documentDB : {e}")

    def get_documents(self, projection=None, filter=None):
        try:
            return self.collection.find(filter=filter, projection=projection)
        except Exception as e:
            print(f"Error while getting data from documentDB : {e}")

    def update_list_in_document(self, filter, list_name, list_value):
        try:
            self.collection.update_one(filter, {"$push": {list_name: list_value}})
        except Exception as e:
            print(f"Error while updating list in documentDB : {e}")

    def delete_document(self, id):
        try:
            self.collection.delete_one({"_id": id})
            print("Document successfully deleted")
        except Exception as e:
            print(f"Error while deleting data from documentDB : {e}")

    def delete_all_documents(self):
        try:
            self.collection.delete_many({})
            print("All documents are successfully deleted")
        except Exception as e:
            print(f"Error while deleting data from documentDB : {e}")


from .odbc import Odbc
from .mssql import Mssql

class SqlFactory:
    @staticmethod
    def create_sql_connector(connector_type, server, database, username, password):
        if connector_type == "odbc":
            return Odbc(server, database, username, password)
        elif connector_type == "mssql":
            return Mssql(server, database, username, password)
        else:
            raise ValueError("Invalid connector type")


import botocore
import boto3
import os
import sys

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.application import MIMEApplication

from cmpparis.parameters_utils import *

def send_email_to_support(subject, content):
    from_email = get_parameter('generic', 'technical_report_email')
    to_email = get_parameter('generic', 'to_support_email')

    send_email(from_email, to_email, subject, content, [])

def send_email(from_email, to_email, subject, data, attachments=None):
    try:
        aws_region_name = get_region_name()
        client = boto3.client('ses', region_name=aws_region_name)

        html = """\
            <html>
                <head><style>th {{ text-align: center; }} td:nth-child(2) {{ font-weight: bold; }}</style></head>
                <body>
                    {}
                </body>
            </html>
        """.format(data)

        msg = MIMEMultipart()

        html_content = MIMEText(html, "html")
        msg.attach(html_content)

        if attachments is not None:
            for file in attachments:
                att = MIMEApplication(open(file, "rb").read())

                att.add_header('Content-Disposition', 'attachment',
                            filename=os.path.basename(file))

                msg.attach(att)

        msg['Subject'] = subject
        msg['From'] = from_email
        msg['To'] = to_email

        response = client.send_raw_email(
            Source=from_email,
            Destinations=to_email.split(','),
            RawMessage={'Data': msg.as_string()}
        )

        return response
    except botocore.exceptions.ClientError as e:
        print(f"Error while sending email : {e}")
        sys.exit(1)


import pytest
import os
from datetime import datetime
from unittest.mock import patch, MagicMock
from cmpparis.document_db_manager import DocumentDBManager

@pytest.fixture
def mock_mongo_client():
    with patch('cmpparis.document_db_manager.pymongo.MongoClient') as mock_client:
        yield mock_client

@pytest.fixture
def db_manager(mock_mongo_client):
    # Instancie DocumentDBManager avec des valeurs fictives
    return DocumentDBManager(
        db_user="test_user",
        db_pwd="test_pwd",
        db_host="test_host",
        database_name="test_db",
        collection_name="test_collection",
        pem_file_path="test.pem"
    )

def test_connect_to_documentdb(mock_mongo_client, db_manager):
    # Récupérer le chemin complet du fichier PEM tel qu'il est utilisé dans le code
    expected_pem_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../cmpparis/global-bundle.pem'))

    # Construire la chaîne de connexion attendue
    expected_uri = (
        f"mongodb://test_user:test_pwd@test_host:27017/"
        f"?tls=true&tlsCAFile={expected_pem_path}&retryWrites=false&directConnection=true"
    )

    # Vérifie que la méthode de connexion est appelée correctement
    mock_mongo_client.assert_called_once_with(expected_uri)

def test_insert_document(db_manager):
    mock_insert = db_manager.collection.insert_one = MagicMock(return_value=MagicMock(inserted_id="test_id"))
    
    data = {"key": "value"}
    data['createdAt'] = datetime.now()
    data['lastModificationAt'] = datetime.now()
    result = db_manager.insert_document(data)
    
    mock_insert.assert_called_once_with(data)
    assert result == "test_id"

def test_update_document(db_manager):
    mock_update = db_manager.collection.update_one = MagicMock(return_value=MagicMock(modified_count=1))
    
    # Appel de la méthode
    result = db_manager.update_document({"key": "value"}, {"key": "new_value"})
    
    # Vérification de l'appel
    mock_update.assert_called_once_with({"key": "value"}, {"$set": {"key": "new_value"}})
    
    # Vérification du résultat
    assert result == 1

def test_get_document(db_manager):
    mock_find = db_manager.collection.find_one = MagicMock(return_value={"key": "value"})
    
    result = db_manager.get_document("key", "value")
    
    mock_find.assert_called_once_with({"key": "value"})
    assert result == {"key": "value"}

def test_get_documents(db_manager):
    mock_find = db_manager.collection.find = MagicMock(return_value=[{"key": "value"}])
    
    result = list(db_manager.get_documents(projection={"key": 1}, filter={"key": "value"}))
    
    mock_find.assert_called_once_with(filter={"key": "value"}, projection={"key": 1})
    assert result == [{"key": "value"}]

def test_update_list_in_document(db_manager):
    mock_update = db_manager.collection.update_one = MagicMock()
    
    db_manager.update_list_in_document({"key": "value"}, "list_field", "new_item")
    
    mock_update.assert_called_once_with({"key": "value"}, {"$push": {"list_field": "new_item"}})

def test_delete_document(db_manager):
    mock_delete = db_manager.collection.delete_one = MagicMock()
    
    db_manager.delete_document("test_id")
    
    mock_delete.assert_called_once_with({"_id": "test_id"})

def test_delete_all_documents(db_manager):
    mock_delete_many = db_manager.collection.delete_many = MagicMock()
    
    db_manager.delete_all_documents()
    
    mock_delete_many.assert_called_once_with({})



import boto3
import os
import sys

from botocore.exceptions import ClientError

from cmpparis.parameters_utils import *
from cmpparis.ses_utils import *

class S3:
    def __init__(self, aws_region_name, aws_bucket_name):
        self.aws_region_name = aws_region_name
        self.aws_bucket_name = aws_bucket_name
        self.s3 = boto3.client('s3', region_name=aws_region_name)

    # Function to download file from S3 bucket
    def download_file_from_s3(self, s3_key, local_filename):
        try:
            self.s3.download_file(self.aws_bucket_name, s3_key, local_filename)
            print("The file has been successfully downloaded from the S3 bucket")
            return True
        except ClientError as e:
            error_message = f"Error while downloading the CSV file from the S3 bucket : {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    # Function to get file from S3 bucket
    def get_file_from_s3(self, s3_key):
        try:
            response = self.s3.get_object(Bucket=self.aws_bucket_name, Key=s3_key)
            return response['Body'].read().decode('utf-8')
        except ClientError as e:
            error_message = f"Error while getting the file from S3 : {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    # Function to get all files from S3 bucket location
    def get_files_from_s3(self, s3_key):
        try:
            response = self.s3.list_objects_v2(Bucket=self.aws_bucket_name, Prefix=s3_key)
            files = []
            for obj in response.get('Contents', []):
                files.append(obj['Key'])
            return files
        except ClientError as e:
            error_message = f"Error while getting the files from S3 : {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    # Function to upload a file to S3
    def upload_file_to_s3(self, local_filename, s3_key = None):
        try:
            if (s3_key is None):
                s3_key = os.path.basename(local_filename)

            self.s3.upload_file(local_filename, self.aws_bucket_name, s3_key)
            print("File uploaded and archived to S3")
            return True
        except ClientError as e:
            error_message = f"Error while uploading file to S3 : {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)

    # Function to delete file from S3 bucket
    def delete_file_from_s3(self, s3_key):
        try:
            self.s3.delete_object(Bucket=self.aws_bucket_name, Key=s3_key)
            print("File deleted from S3")
            return True
        except ClientError as e:
            error_message = f"Error while deleting file from S3 : {e}"
            print(error_message)
            send_email_to_support(error_message)

            sys.exit(1)


import requests
import unittest
from unittest.mock import patch, Mock
from cmpparis.quable_api import QuableAPI

class TestQuableAPI(unittest.TestCase):
    def setUp(self):
        self.quable_api = QuableAPI()

    @patch('cmpparis.quable_api.requests.get')
    def test_get(self, mock_get):
        mock_response = Mock()
        mock_response.json.return_value = {'data': 'test_data'}
        mock_response.status_code = 200
        mock_get.return_value = mock_response

        response = self.quable_api.get('test_endpoint')
        self.assertEqual(response, {'data': 'test_data'})
        mock_get.assert_called_once_with(f'{self.quable_api.base_url}/test_endpoint', headers=self.quable_api.headers, params=None)

    @patch('cmpparis.quable_api.requests.get')
    def test_get_with_params(self, mock_get):
        mock_response = Mock()
        mock_response.json.return_value = {'data': 'test_data'}
        mock_response.status_code = 200
        mock_get.return_value = mock_response

        params = {'param1': 'value1', 'param2': 'value2'}
        response = self.quable_api.get('test_endpoint', params=params)
        self.assertEqual(response, {'data': 'test_data'})
        mock_get.assert_called_once_with(f'{self.quable_api.base_url}/test_endpoint', headers=self.quable_api.headers, params=params)

    @patch('cmpparis.quable_api.requests.post')
    def test_post(self, mock_post):
        mock_response = Mock()
        mock_response.json.return_value = {'data': 'test_data'}
        mock_response.status_code = 200
        mock_post.return_value = mock_response

        data = {'key': 'value'}
        response = self.quable_api.post('test_endpoint', data)
        self.assertEqual(response, {'data': 'test_data'})
        mock_post.assert_called_once_with(f'{self.quable_api.base_url}/test_endpoint', headers=self.quable_api.headers, json=data)

if __name__ == '__main__':
    unittest.main()


from .sql_interface import SqlInterface

import pyodbc

class Odbc(SqlInterface):
    def __init__(self, server, database, username, password, port = None):
        self.server = server
        self.database = database
        self.username = username
        self.password = password
        if port is not None:
            self.port = port

    def connect(self, driver):
        try:
            connection_string = "DRIVER={{{driver}}};SERVER={server};DATABASE={database};UID={username};PWD={password};".format(
                driver=driver,
                server=self.server,
                database=self.database,
                username=self.username,
                password=self.password
            )

            return pyodbc.connect(connection_string)
        except Exception as e:
            print(f"Error connecting to SQL Server: {e}")

    def disconnect(self, connection):
        try:
            connection.close()
        except Exception as e:
            print(f"Error disconnecting from SQL Server: {e}")